import gql from 'graphql-tag'
import { graphql } from 'react-apollo'
import { compose, withHandlers, withProps } from 'recompose'

const userFragment = gql`
  fragment userDetails on User {
    id
    admin
    email
    title
    country
    username
    lastName
    isActive
    firstName
    affiliation
    editorInChief
    handlingEditor
  }
`

const getUsersQuery = gql`
  {
    users {
      ...userDetails
    }
  }
  ${userFragment}
`

const addUserAsAdmin = gql`
  mutation addUserAsAdmin($id: ID!, $input: UserInput!) {
    addUserAsAdmin(id: $id, input: $input) {
      ...userDetails
    }
  }
  ${userFragment}
`

const editUserAsAdmin = gql`
  mutation editUserAsAdmin($id: ID!, $input: UserInput!) {
    editUserAsAdmin(id: $id, input: $input) {
      ...userDetails
    }
  }
  ${userFragment}
`

const activateUserAsAdmin = gql`
  mutation activateUserAsAdmin($id: ID!, $input: ActivateUserInput) {
    activateUserAsAdmin(id: $id, input: $input) {
      ...userDetails
    }
  }
  ${userFragment}
`

export default compose(
  graphql(getUsersQuery),
  graphql(addUserAsAdmin, {
    name: 'addUser',
    options: {
      refetchQueries: [{ query: getUsersQuery }],
    },
  }),
  graphql(editUserAsAdmin, {
    name: 'updateUser',
  }),
  graphql(activateUserAsAdmin, {
    name: 'activateUser',
  }),
  withHandlers({
    addUser: ({ addUser }) => (
      { __typename, id, admin, ...input },
      { props: { hideModal, setFetching } },
    ) => {
      setFetching(true)
      addUser({
        variables: {
          id,
          input: {
            ...input,
            username: input.email,
          },
        },
      }).then(() => {
        setFetching(false)
        hideModal()
      })
    },
    updateUser: ({ updateUser }) => (
      { __typename, id, admin, handlingEditor, editorInChief, ...input },
      { props: { hideModal, setFetching, setError } },
    ) => {
      setFetching(true)
      updateUser({
        variables: {
          id,
          input,
        },
      })
        .then(() => {
          setFetching(false)
          hideModal()
        })
        .catch(e => {
          setFetching(false)
          setError(e.message)
        })
    },
    toggleUserActiveStatus: ({ activateUser }) => ({
      __typename,
      admin,
      handlingEditor,
      editorInChief,
      id,
      email,
      username,
      isActive,
    }) => ({ hideModal }) => {
      activateUser({
        variables: {
          id,
          input: {
            email,
            username,
            isActive,
          },
        },
      }).then(hideModal)
    },
  }),
  withProps(({ data }) => ({
    users: data.users,
  })),
)
